# this code is based on cpan2rpm:
#   cpan2rpm - CPAN module RPM maker
#   Copyright (C) 2001-2003 Erick Calder
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

package RPM::Source::BundleImport::Perl::CPAN2RPM_get_meta;

use strict;
use warnings;

use Cwd;
use Carp;
use File::Spec;
use ExtUtils::MM_Unix;
# used as is. it passes to WriteMakefile - so it is ok.
use ExtUtils::Depends;

use RPM::Source::BundleImport::Perl::Utils;

### TODO!!!
our $opt_eval_keep_pipes=1;

sub cpan2rpm_get_meta {
    my ($extractdir) = @_;
    my $info = {};
    my $CWD = getcwd;
    eval {
	&cpan2rpm_body($extractdir,$info);
    };
    chdir $CWD;
    die "cpan2rpm_get_meta: $@" if $@;
    return $info;
}

sub cpan2rpm_body {
    my ($rel_extractdir,$info) = @_;
    my %meta;
    my $CWD = getcwd;
    local $_;

    my $extractdir = File::Spec->rel2abs( $rel_extractdir );

    # sometimes extracted dir is read-only
    chmod 0775, $rel_extractdir;
    chdir $rel_extractdir || die "get_meta(): cd $rel_extractdir: $!";

    # AutoInstall support
    $ENV{'AUTOMATED_TESTING'}=1;

    $_ = "$extractdir/Build.PL";
    $_ = "$extractdir/Makefile.PL" unless -e;
    die qq/No PL file [$_] in tarball/ unless -e;
    die qq/Cannot read PL file [$_]/ unless -r;

    my $cpan_install=/CPAN::install/;
    $cpan_install=1 if /CPANPLUS::install/;;
    if ($cpan_install and not /ExtUtils::AutoInstall/) {
	print <<'EMSG';
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!							!!
!!							!!
!!		TODO: FAILED:	CPAN::install		!!
!!							!!
!!	Oops! FATAL! fatal! ERROR! error! HURRAY!	!!
!!							!!
!!		a CPAN::install found!			!!
!!							!!
!!							!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
EMSG
	return;
    }


    my $infoPL;
    ($infoPL = $_) =~ s|.*/||;

    #   we want to protect us from exit()ing but without modifying the
    #   actual Makefile.PL since we may be operating in a source directory

    my $PL = readfile($infoPL);
    $PL =~ s/\bexit\b/return/gs;
    $PL =~ s/while\s*\(\s*\<(?:STDIN)?\>\s*\)/while (0)/;
    $PL =~ s/while\s*\(\s*1\s*\)/while (0)/;
    $PL =~ s/([\(=\s])\<(?:STDIN)?\>([\s\);}])/${1}'n'${2}/;
    $PL =~ s/use\s+(?:CPAN|Class::HPLOO|Inline)::MakeMaker/use ExtUtils::MakeMaker/;
    # inc/Module/Install specific
    $PL =~ s/(^|;|\s)auto_install($|;|\s)/$1$2/;
    
    my $t = "$extractdir/PL.$$";
    #print STDERR $PL;exit;
    writefile($t, $PL);
    unlink ("$extractdir/inc/Module/Install/AutoInstall.pm");
    my @fixfiles=grep {-e $_} map {"$extractdir/$_"} qw!inc/MyBuild.pm!;
    (system ('sed', '-i', '-e','s,\bexit\b,return,g',@fixfiles)==0 or warn "sed for @fixfiles failed") if @fixfiles;
	

    #   now we get the arguments passed to the method that creates
    #   the make script (WriteMakefile || create_build_script)

    {
    no warnings;

    #   dynamically load the module (so when the PL file loads it
    #   we already own it and have hijacked the appropriate method

    my $PLMOD = "ExtUtils::MakeMaker";
    if ($infoPL =~ /^Build/) {
        $PLMOD = "Module::Build"; 
        }
    $@ = ""; eval "use $PLMOD"; die "$PLMOD unloadable\n $@" if $@;

    # grab parameters to function e.g. WriteMakefile()
    $ENV{'_DEEP_RECURSION'}=0;
    if ($PLMOD eq "ExtUtils::MakeMaker") {
	my $PLFN = "ExtUtils::MakeMaker::WriteMakefile";
    eval qq/*${PLFN}_orig = \\&$PLFN/;
    eval qq/*$PLFN = sub {
        die "SAFETY ABORT!" if \$ENV{_DEEP_RECURSION}++ > 10;
        %meta = \@_ unless %meta;
        *$PLFN = \\&${PLFN}_orig;
        goto &$PLFN;
        };/;
    } elsif ($PLMOD eq "Module::Build") {
	my $PLFN = "Module::Build::new";
    eval qq/*${PLFN}_orig = \\&$PLFN/;
    eval qq/*$PLFN = sub {
        die "SAFETY ABORT!" if \$ENV{_DEEP_RECURSION}++ > 10;
        shift; # class name
        %meta = \@_ unless %meta;
        *$PLFN = \\&${PLFN}_orig;
        goto &$PLFN;
        };/;
    }
    # TODO: repeat eval hack above.
    # prompt killer
    $ENV{PERL_MM_USE_DEFAULT}=1;
    #*ExtUtils::MakeMaker::prompt=sub ($;$) {print "interactive prompt:$_[0]\n$_[1]\n" ;return $_[1] eq 'y' ? 'n' : $_[1]};
    #*Module::Build::prompt=sub {print "interactive prompt:$_[0]\n$_[1]\n" ;return $_[2] eq 'y' ? 'n' : $_[2]};
    #*Module::Build::y_n=sub {print "interactive y_n:$_[0]\n" ;return $_[1] eq 'y' ? 'n' : $_[1]};

    local @ARGV = ();
    local $0 = $t;
    # no input or output
    std_close();
    # execute the makefile
    my $ok = 0;
    eval {
	#against Filter-Rijndael
	local $/;
	local $SIG{__DIE__}='DEFAULT';
	local $SIG{__WARN__}='DEFAULT';
        do $t; $ok++;
        };
    my $err = $@;
    # clean up
    std_restore();
    #against Filter-Rijndael
    #$/="\n";
    # against Tripletail
    # local %SIG does not seem to work
    delete $SIG{__DIE__};#='DEFAULT';
    delete $SIG{__WARN__};

    unlink $t || die "get_meta(): rm[$t] - $!";
    if (!$ok) {
        $err =~ s/$t/$infoPL/g;
        die "FATAL CRASH! Could not load $infoPL:\n$err";
        }

    #   map Build.PL hash keys to MakeMaker's
 
#       meta_add => {
#                resources => {
#                        homepage => 'https://github.com/ThePilgrim/perlcldr',
 


    if ($infoPL =~ /^Build/) {
        my %b2m = qw/
            build_requires BUILD_REQUIRES
            configure_requires CONFIGURE_REQUIRES
            recommends RECOMMENDS
            requires PREREQ_PM
            module_name NAME
            dist_name DISTNAME
            dist_abstract ABSTRACT
            dist_author AUTHOR
            dist_version VERSION
            dist_version_from VERSION_FROM
            license LICENSE
            meta_add META_ADD
            meta_merge META_MERGE
            xs_files XS_FILES
            /;
        $meta{$b2m{$_}} = $meta{$_} for keys %b2m;
        }
    }

    #   figure out package name
    my $module = &__NAME_to_path($meta{NAME} || $meta{DISTNAME});

    #   extract version
    $info->{version} ||= $meta{VERSION};
    if (not defined($info->{version}) and $meta{VERSION_FROM}) {
        $info->{version} = ExtUtils::MM_Unix->parse_version($meta{VERSION_FROM});
        trim($info->{version}); # parse_version() returns spaces
    }

    #   get module description info
    $info->{abstract} ||= $meta{ABSTRACT};
    if (not $info->{abstract} and $meta{ABSTRACT_FROM}) {
	$meta{DISTNAME}||=&__path_to_name($meta{ABSTRACT_FROM});
	$meta{DISTNAME}||=&__path_to_name($module);
	my $fake_ExtUtils_MM_Unix=\%meta;
	bless $fake_ExtUtils_MM_Unix, 'ExtUtils::MM_Unix'; # for $meta{DISTNAME}
        $info->{abstract} = $fake_ExtUtils_MM_Unix->parse_abstract($meta{ABSTRACT_FROM});
        trim($info->{abstract});
    }
    foreach my $description_from ($meta{ABSTRACT_FROM}, $meta{VERSION_FROM}, $module) {
	if (defined($description_from) and -e $description_from) {
	    &RPM::Source::BundleImport::Perl::Utils::update_metadata_abstract_description($info,
	    &RPM::Source::BundleImport::Perl::Utils::get_abstract_description_from_pod($description_from));
	}
    }

    # return to user's directory
    chdir $CWD;

    #use Data::Dumper;
    #print "DEBUG: raw metadata at cpan2rpm:\n", Dumper(\%meta),"\n";# if $verbose > 2;

    $info->{'requires'}=$meta{PREREQ_PM};
    $info->{'requires'}||=$meta{'prereq'};
    $info->{'build_requires'}=$meta{BUILD_REQUIRES};
    $info->{'configure_requires'}=$meta{CONFIGURE_REQUIRES};
    $info->{'recommends'}=$meta{RECOMMENDS};
    $info->{'scripts'}=$meta{EXE_FILES};
    $info->{'scripts'}||= $meta{'script_files'};
    $info->{'scripts'}||= $meta{'scripts'};
    $info->{'man1pods'}||=$meta{MAN1PODS};
    # TODO: use!
    $info->{'license'}=$meta{LICENSE};
    # TODO: meta_add/meta_merge homepage

    # TODO: make use of those fields
    # XS_FILES
    # found in Module::Build
    #    extra_compiler_flags
    #    extra_linker_flags

    foreach my $i (qw/XSOPT CC LD LIBS
/) {
	$info->{$i}||=$meta{$i};
    }
    return $info;
}


sub __NAME_to_path {
    my $name=shift;
    return unless defined $name;
    my $pm=$name.'.pm';
    $pm=~s!::!/!g;
    if (-e 'lib/'.$pm) {
	return 'lib/'.$pm;
    } elsif (-e $pm) {
	return $pm;
    }
    $pm=$name.'.pm';
    $pm=~s!.*::!!g;
    return $pm;
}

sub __path_to_name {
    my $pm=shift;
    return unless defined $pm;
    $pm=~s!^lib/!!g;
    $pm=~s!\.pm$!!g;
    $pm=~s!/!-!g;
    return $pm;
}


# --- file handling functions -------------------------------------------------

#
#   returns the contents of a given file or undef if the
#   when called in void context, sets $_
#

sub readfile {
    my $f = shift;
    confess "readfile: undefined f" unless defined $f;
    local $_;
    return $_ = "" unless -r $f;
    local $/ = undef;
    open(F, $f) || die "$! [$f].  Stopped ";
    $_ = <F>;
    close(F);
    return $_;
    }

#
#    writes a file, from a string
#

sub writefile($@) {
    my $fn = shift;
    local $_ = shift || $_;
    my $op = shift || ">";

    open (FILE, "$op $fn") || die "writefile('$fn'): $!. Stopped";
    binmode(FILE);
    print FILE;
    close FILE || die "writefile('$fn'): $!. Stopped";
    $fn;
    }

sub stub_Name_OLDIN_used_only_once {
    open(OLDIN,  "<&STDIN");
    open(OLDOUT, ">&STDOUT");
    open(OLDERR, ">&STDERR");
}

sub std_close {
    return if $opt_eval_keep_pipes;
    #return if $debug} || $info{"keep-pipes"};
    open(OLDOUT, ">&STDOUT")   || die "std_save(): $!";
    open(OLDERR, ">&STDERR")   || die "std_save(): $!";
    open(OLDIN,  "<&STDIN")    || die "std_save(): $!";
    open(STDOUT, ">/dev/null") || die "std_close(): $!";
    open(STDERR, ">/dev/null") || die "std_close(): $!";
    open(STDIN,  "</dev/null") || die "std_close(): $!";
    }

sub std_restore {
    return if $opt_eval_keep_pipes;
    #return if $info{debug} || $info{"keep-pipes"};
    open(STDOUT, '>&',OLDOUT) || die "std_restore(): $!";
    open(STDERR, '>&',OLDERR) || die "std_restore(): $!";
    open(STDIN,  "<&OLDIN")  || die "std_restore(): $!";
    }

sub trim {
    map { $_ ||= ""; s/^\s+//; s/\s+$//; } @_;
    unless (defined wantarray()) {
        $_ ||= ""; s/^\s+//; s/\s+$//;
        }
    $_[0] if @_ == 1;
    }
