# Copyright (c) 2003-2004, 2012-2013 by the gtk2-perl team (see the file
# AUTHORS)
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Library General Public License for more details.
#
# You should have received a copy of the GNU Library General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA

package ExtUtils::PkgConfig;

use strict;

use vars qw/ $VERSION $AUTOLOAD/;

$VERSION = '2.15';

our @pkgconfig;

sub clean {
    @pkgconfig=();
}

sub find {
	my ($class, @pkg_candidates) = @_;
	foreach my $candidate (@pkg_candidates) {
	    $candidate=~s/\s+[=<>!].*$//; # "gstreamer-gconf-0.8 >= $build_reqs{'GStreamerGConf'}"
	    #print STDERR "ExtUtils::PkgConfig: candidate $candidate\n";
	    push @pkgconfig, $candidate;
	}
#			croak "*** can not find package for any of ($list)\n"
#			    . "*** check that one of them is properly installed and available in PKG_CONFIG_PATH\n";
#			croak "*** can not find package $pkg_candidates[0]\n"
#			    . "*** check that it is properly installed and available in PKG_CONFIG_PATH\n";
	my %data = ();
#		croak "*** can't find $what for \"$data{pkg}\"\n"
#		    . "*** is it properly installed and available in PKG_CONFIG_PATH?\n"
	return %data;
}

sub create_version_macros {}
sub write_version_macros {}

sub modversion{ push @pkgconfig, $_[0]}
sub libs{ push @pkgconfig, $_[0]}
sub cflags{ push @pkgconfig, $_[0]}
sub cflags_only_I{ push @pkgconfig, $_[0]}
sub cflags_only_other{ push @pkgconfig, $_[0]}
sub libs_only_L{ push @pkgconfig, $_[0]}
sub libs_only_l{ push @pkgconfig, $_[0]}
sub libs_only_other{ push @pkgconfig, $_[0]}
sub static_libs{ push @pkgconfig, $_[0]}
sub variable{ push @pkgconfig, $_[0]}
sub atleast_version{ push @pkgconfig, $_[0]}
sub exact_version{ push @pkgconfig, $_[0]}
sub max_version{ push @pkgconfig, $_[0]}

1;

=head1 NAME

ExtUtils::PkgConfig - simplistic interface to pkg-config

=head1 SYNOPSIS

 use ExtUtils::PkgConfig;

 $package = 'gtk+-2.0';

 %pkg_info = ExtUtils::PkgConfig->find ($package);
 print "modversion:  $pkg_info{modversion}\n";
 print "cflags:      $pkg_info{cflags}\n";
 print "libs:        $pkg_info{libs}\n";


=head1 DESCRIPTION

The pkg-config program retrieves information about installed libraries,
usually for the purposes of compiling against and linking to them.

ExtUtils::PkgConfig is a very simplistic interface to this utility, intended
for use in the Makefile.PL of perl extensions which bind libraries that
pkg-config knows.  It is really just boilerplate code that you would've
written yourself.

=head2 USAGE

=over

=item HASH = ExtUtils::PkgConfig->find (STRING, [STRING, ...])

Call pkg-config on the library specified by I<STRING> (you'll have to know what
to use here).  The returned I<HASH> contains the modversion, cflags, and libs
values under keys with those names. If multiple STRINGS are passed they are
attempted in the order they are given till a working package is found.

If pkg-config fails to find a working I<STRING>, this function croaks with a
message intended to be helpful to whomever is attempting to compile your
package.

For example:

  *** can not find package bad1
  *** check that it is properly installed and available
  *** in PKG_CONFIG_PATH

or

  *** can't find cflags for gtk+-2.0
  *** is it properly installed and available in PKG_CONFIG_PATH?

=item STRING = ExtUtils::PkgConfig->create_version_macros (PACKAGE, STEM)

Create a set of version macros with the prefix I<STEM> for the library
specified by I<PACKAGE>.  The result is returned.

Example input would be "gtk+-2.0" for I<PACKAGE> and "GTK" for I<STEM>.

=item ExtUtils::PkgConfig->write_version_macros (FILE, PACKAGE, STEM, [PACKAGE, STEM, ...])

Create one or more sets of version macros for the libraries and prefixes
specified by the I<PACKAGE> and I<STEM> pairs and write them to the file
I<FILE>.  If it doesn't exist, I<FILE> will be created.  If it does exist, it
will be overwritten.

=back

=head1 SEE ALSO

ExtUtils::PkgConfig was designed to work with ExtUtils::Depends for compiling
the various modules of the gtk2-perl project.

  L<ExtUtils::Depends>

  L<http://gtk2-perl.sourceforge.net/>

This module is really just an interface to the pkg-config utility program.
http://www.freedesktop.org/Software/pkgconfig

=head1 AUTHORS

muppet E<lt>scott at asofyet dot orgE<gt>.

=head1 COPYRIGHT AND LICENSE

Copyright 2003-2004, 2012-2013 by muppet, Ross McFarland, and the gtk2-perl
team

This library is free software; you can redistribute it and/or modify
it under the terms of the Lesser General Public License (LGPL).  For
more information, see http://www.fsf.org/licenses/lgpl.txt

=cut
